import pytest


def test_FluentMessage___init__():
    assert True


def test_FluentMessage_attach():
    assert True


def test_FluentMessage_body():
    assert True


def test_FluentMessage_from_():
    assert True


def test_FluentMessage_send():
    assert True


def test_FluentMessage_sign():
    assert True


def test_FluentMessage_subject():
    assert True


def test_FluentMessage_to():
    assert True
